@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2021 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -149,8 +149,6 @@ abstract class CControllerPopupItemTest extends CController {
 			'support_lld_macros' => true
 		],
 		'http_proxy' => [
-			'host' => ['{HOSTNAME}', '{HOST.HOST}', '{HOST.NAME}'],
-			'interface' => ['{HOST.IP}', '{IPADDRESS}', '{HOST.DNS}', '{HOST.CONN}', '{HOST.PORT}'],
 			'item' => ['{ITEM.ID}', '{ITEM.KEY.ORIG}', '{ITEM.KEY}'],
 			'support_user_macros' => true,
 			'support_lld_macros' => true
@@ -247,9 +245,9 @@ abstract class CControllerPopupItemTest extends CController {
 	protected $is_item_testable;
 
 	/**
-	 * @var object
+	 * @var string
 	 */
-	protected $preproc_item;
+	protected $test_type;
 
 	/**
 	 * @var array
@@ -331,26 +329,6 @@ abstract class CControllerPopupItemTest extends CController {
 		return $ret;
 	}
 
-	/**
-	 * Function returns instance of item, item prototype or discovery rule class.
-	 *
-	 * @param int $test_type
-	 *
-	 * @return CItem|CItemPrototype|CDiscoveryRule
-	 */
-	protected static function getPreprocessingItemClassInstance($test_type) {
-		switch ($test_type) {
-			case self::ZBX_TEST_TYPE_ITEM:
-				return new CItem;
-
-			case self::ZBX_TEST_TYPE_ITEM_PROTOTYPE:
-				return new CItemPrototype;
-
-			case self::ZBX_TEST_TYPE_LLD:
-				return new CDiscoveryRule;
-		}
-	}
-
 	/**
 	 * Function returns list of proxies.
 	 *
@@ -376,7 +354,7 @@ abstract class CControllerPopupItemTest extends CController {
 	 * Function returns array of item specific properties used for item testing.
 	 *
 	 * @param array $input       Stored user input used to overwrite values retrieved from database.
-	 * @param bool  $for_server  Whether need to add to result an additional properties used only for connection with
+	 * @param bool  $for_server  Whether need to add to result additional properties used only for connection with
 	 *                           Zabbix server.
 	 *
 	 * @return array
@@ -563,7 +541,7 @@ abstract class CControllerPopupItemTest extends CController {
 					'key' => $input['key'],
 					'http_authtype' => array_key_exists('http_authtype', $input)
 						? $input['http_authtype']
-						: HTTPTEST_AUTH_NONE,
+						: ZBX_HTTP_AUTH_NONE,
 					'follow_redirects' => array_key_exists('follow_redirects', $input) ? $input['follow_redirects'] : 0,
 					'headers' => array_key_exists('headers', $input) ? $input['headers'] : [],
 					'http_proxy' => array_key_exists('http_proxy', $input) ? $input['http_proxy'] : null,
@@ -589,7 +567,7 @@ abstract class CControllerPopupItemTest extends CController {
 					'verify_peer' => array_key_exists('verify_peer', $input) ? $input['verify_peer'] : 0
 				];
 
-				if ($data['http_authtype'] != HTTPTEST_AUTH_NONE) {
+				if ($data['http_authtype'] != ZBX_HTTP_AUTH_NONE) {
 					$data += [
 						'http_username' => array_key_exists('http_username', $input) ? $input['http_username'] : null,
 						'http_password' => array_key_exists('http_password', $input) ? $input['http_password'] : null
@@ -719,7 +697,8 @@ abstract class CControllerPopupItemTest extends CController {
 				'privpassphrase' => '',
 				'authprotocol' => ITEM_SNMPV3_AUTHPROTOCOL_MD5,
 				'privprotocol' => ITEM_SNMPV3_PRIVPROTOCOL_DES,
-				'contextname' => ''
+				'contextname' => '',
+				'max_repetitions' => '10'
 			]
 		];
 
@@ -737,7 +716,13 @@ abstract class CControllerPopupItemTest extends CController {
 				$output[] = 'details';
 			}
 
-			if (itemTypeInterface($this->item_type) === false) {
+			$item_type_interface = itemTypeInterface($this->item_type);
+
+			if ($item_type_interface == INTERFACE_TYPE_OPT && $inputs['interfaceid'] == 0) {
+				$item_type_interface = false;
+			}
+
+			if ($item_type_interface === false) {
 				$host_interfaces = API::HostInterface()->get([
 					'output' => $output,
 					'hostids' => $this->host['hostid'],
@@ -745,11 +730,7 @@ abstract class CControllerPopupItemTest extends CController {
 				]);
 				$host_interfaces = zbx_toHash($host_interfaces, 'type');
 
-				$ordered_interface_types = [INTERFACE_TYPE_AGENT, INTERFACE_TYPE_SNMP, INTERFACE_TYPE_JMX,
-					INTERFACE_TYPE_IPMI
-				];
-
-				foreach ($ordered_interface_types as $interface_type) {
+				foreach (CItem::INTERFACE_TYPES_BY_PRIORITY as $interface_type) {
 					if (array_key_exists($interface_type, $host_interfaces)) {
 						$interfaces[] = $host_interfaces[$interface_type];
 						break;
@@ -927,7 +908,7 @@ abstract class CControllerPopupItemTest extends CController {
 				'supported_macros' => array_diff_key($this->macros_by_item_props['key'],
 					['support_user_macros' => true, 'support_lld_macros' => true]
 				),
-				'support_lldmacros' => ($this->preproc_item instanceof CItemPrototype),
+				'support_lldmacros' => ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE),
 				'texts_support_macros' => [$inputs['key']],
 				'texts_support_lld_macros' => [$inputs['key']],
 				'texts_support_user_macros' => [$inputs['key']],
@@ -1023,7 +1004,7 @@ abstract class CControllerPopupItemTest extends CController {
 	protected function resolvePreprocessingStepMacros(array $steps) {
 		// Resolve macros used in parameter fields.
 		$macros_posted = $this->getInput('macros', []);
-		$macros_types = ($this->preproc_item instanceof CItemPrototype)
+		$macros_types = ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE)
 			? ['usermacros' => true, 'lldmacros' => true]
 			: ['usermacros' => true];
 
@@ -1036,7 +1017,7 @@ abstract class CControllerPopupItemTest extends CController {
 
 			// Resolve macros in parameter fields before send data to Zabbix server.
 			foreach (['params', 'error_handler_params'] as $field) {
-				$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($step[$field], $macros_types);
+				$matched_macros = CMacrosResolverGeneral::getMacroPositions($step[$field], $macros_types);
 
 				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 					$macro_value = array_key_exists($macro, $macros_posted)
@@ -1055,20 +1036,19 @@ abstract class CControllerPopupItemTest extends CController {
 	/**
 	 * Resolve macros used in the calculates item formula.
 	 *
-	 * @param string $formula  Calculated item formula.
+	 * @param string $formula        Calculated item formula.
+	 * @param array  $macros_posted  Macros.
 	 *
-	 * @return array
+	 * @return string
 	 */
-	private function resolveCalcFormulaMacros(string $formula) {
-		$macros_posted = $this->getInput('macros', []);
-
+	private function resolveCalcFormulaMacros(string $formula, array $macros_posted): string {
 		if (!$macros_posted) {
 			return $formula;
 		}
 
 		$expression_parser = new CExpressionParser([
 			'usermacros' => true,
-			'lldmacros' => ($this->preproc_item instanceof CItemPrototype),
+			'lldmacros' => ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE),
 			'calculated' => true,
 			'host_macro' => true,
 			'empty_host' => true
@@ -1088,51 +1068,79 @@ abstract class CControllerPopupItemTest extends CController {
 			CExpressionParserResult::TOKEN_TYPE_STRING,
 			CExpressionParserResult::TOKEN_TYPE_HIST_FUNCTION
 		]);
+
 		foreach ($tokens as $token) {
+			if ($pos_left != $token['pos']) {
+				$expression[] = substr($formula, $pos_left, $token['pos'] - $pos_left);
+			}
+			$pos_left = $token['pos'] + $token['length'];
+
 			switch ($token['type']) {
 				case CExpressionParserResult::TOKEN_TYPE_USER_MACRO:
 				case CExpressionParserResult::TOKEN_TYPE_LLD_MACRO:
-					if ($pos_left != $token['pos']) {
-						$expression[] = substr($formula, $pos_left, $token['pos'] - $pos_left);
-					}
-					$pos_left = $token['pos'] + $token['length'];
-
 					$expression[] = array_key_exists($token['match'], $macros_posted)
 						? CExpressionParser::quoteString($macros_posted[$token['match']], false)
 						: $token['match'];
 					break;
 
 				case CExpressionParserResult::TOKEN_TYPE_STRING:
-					if ($pos_left != $token['pos']) {
-						$expression[] = substr($formula, $pos_left, $token['pos'] - $pos_left);
-					}
-					$pos_left = $token['pos'] + $token['length'];
-
 					$string = strtr(CExpressionParser::unquoteString($token['match']), $macros_posted);
 					$expression[] = CExpressionParser::quoteString($string, false, true);
 					break;
 
 				case CExpressionParserResult::TOKEN_TYPE_HIST_FUNCTION:
-					foreach ($token['data']['parameters'][0]['data']['filter']['tokens'] as $filter_token) {
-						switch ($filter_token['type']) {
-							case CFilterParser::TOKEN_TYPE_USER_MACRO:
-							case CFilterParser::TOKEN_TYPE_LLD_MACRO:
-								if ($pos_left != $filter_token['pos']) {
-									$expression[] = substr($formula, $pos_left, $filter_token['pos'] - $pos_left);
-								}
-								$pos_left = $filter_token['pos'] + $filter_token['length'];
+					$pos_left = $token['pos'];
 
-								$string = strtr($filter_token['match'], $macros_posted);
+					foreach ($token['data']['parameters'] as $parameter) {
+						if ($pos_left != $parameter['pos']) {
+							$expression[] = substr($formula, $pos_left, $parameter['pos'] - $pos_left);
+						}
+						$pos_left = $parameter['pos'] + $parameter['length'];
+
+						switch ($parameter['type']) {
+							case CHistFunctionParser::PARAM_TYPE_QUERY:
+								$pos_left = $parameter['pos'];
+
+								foreach ($parameter['data']['filter']['tokens'] as $filter_token) {
+									if ($pos_left != $filter_token['pos']) {
+										$expression[] = substr($formula, $pos_left, $filter_token['pos'] - $pos_left);
+									}
+									$pos_left = $filter_token['pos'] + $filter_token['length'];
+
+									switch ($filter_token['type']) {
+										case CFilterParser::TOKEN_TYPE_USER_MACRO:
+										case CFilterParser::TOKEN_TYPE_LLD_MACRO:
+											$string = strtr($filter_token['match'], $macros_posted);
+											$expression[] = CFilterParser::quoteString($string);
+											break;
+
+										case CFilterParser::TOKEN_TYPE_STRING:
+											$string = strtr(CFilterParser::unquoteString($filter_token['match']),
+												$macros_posted
+											);
+											$expression[] = CFilterParser::quoteString($string);
+											break;
+
+										case CFilterParser::TOKEN_TYPE_KEYWORD:
+										case CFilterParser::TOKEN_TYPE_OPERATOR:
+											$expression[] = $filter_token['match'];
+											break;
+									}
+								}
+								break;
+
+							case CHistFunctionParser::PARAM_TYPE_PERIOD:
+								$string = strtr($parameter['match'], $macros_posted);
+								$expression[] = $string;
+								break;
+
+							case CHistFunctionParser::PARAM_TYPE_QUOTED:
+								$string = strtr(CFilterParser::unquoteString($parameter['match']), $macros_posted);
 								$expression[] = CFilterParser::quoteString($string);
 								break;
 
-							case CFilterParser::TOKEN_TYPE_STRING:
-								if ($pos_left != $filter_token['pos']) {
-									$expression[] = substr($formula, $pos_left, $filter_token['pos'] - $pos_left);
-								}
-								$pos_left = $filter_token['pos'] + $filter_token['length'];
-
-								$string = strtr(CFilterParser::unquoteString($filter_token['match']), $macros_posted);
+							case CHistFunctionParser::PARAM_TYPE_UNQUOTED:
+								$string = strtr($parameter['match'], $macros_posted);
 								$expression[] = CFilterParser::quoteString($string);
 								break;
 						}
@@ -1176,7 +1184,7 @@ abstract class CControllerPopupItemTest extends CController {
 				'macros_n' => []
 			];
 
-			if ($this->preproc_item instanceof CItemPrototype) {
+			if ($this->test_type == self::ZBX_TEST_TYPE_ITEM_PROTOTYPE) {
 				$types += ['lldmacros' => true];
 			}
 
@@ -1196,7 +1204,7 @@ abstract class CControllerPopupItemTest extends CController {
 					foreach (array_keys($inputs[$field][$key]) as $nr) {
 						$str = &$inputs[$field][$key][$nr];
 						if (strstr($str, '{') !== false) {
-							$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($str, $types);
+							$matched_macros = CMacrosResolverGeneral::getMacroPositions($str, $types);
 
 							foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 								$macro_value = array_key_exists($macro, $macros_posted)
@@ -1212,25 +1220,30 @@ abstract class CControllerPopupItemTest extends CController {
 				}
 			}
 			elseif (strstr($inputs[$field], '{') !== false) {
-				$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($inputs[$field], $types);
+				if ($field === 'key') {
+					$inputs[$field] = CMacrosResolverGeneral::resolveItemKeyMacros($inputs[$field], $macros_posted, $types);
+				}
+				else {
+					$matched_macros = CMacrosResolverGeneral::getMacroPositions($inputs[$field], $types);
 
-				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
-					$macro_value = array_key_exists($macro, $macros_posted)
-						? $macros_posted[$macro]
-						: '';
+					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+						$macro_value = array_key_exists($macro, $macros_posted)
+							? $macros_posted[$macro]
+							: '';
 
-					if ($inputs['type'] == ITEM_TYPE_HTTPAGENT && $field === 'posts') {
-						if ($inputs['post_type'] == ZBX_POSTTYPE_JSON && !is_numeric($macro_value)) {
-							$macro_value = json_encode($macro_value, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
-							// Remove " wrapping.
-							$macro_value = substr($macro_value, 1, -1);
-						}
-						elseif ($inputs['post_type'] == ZBX_POSTTYPE_XML) {
-							$macro_value = htmlentities($macro_value);
+						if ($inputs['type'] == ITEM_TYPE_HTTPAGENT && $field === 'posts') {
+							if ($inputs['post_type'] == ZBX_POSTTYPE_JSON && !is_numeric($macro_value)) {
+								$macro_value = json_encode($macro_value, JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE);
+								// Remove " wrapping.
+								$macro_value = substr($macro_value, 1, -1);
+							}
+							elseif ($inputs['post_type'] == ZBX_POSTTYPE_XML) {
+								$macro_value = htmlentities($macro_value);
+							}
 						}
+
+						$inputs[$field] = substr_replace($inputs[$field], $macro_value, $pos, strlen($macro));
 					}
-
-					$inputs[$field] = substr_replace($inputs[$field], $macro_value, $pos, strlen($macro));
 				}
 			}
 		}
@@ -1239,12 +1252,13 @@ abstract class CControllerPopupItemTest extends CController {
 		if (array_key_exists('interface', $inputs) && array_key_exists('details', $inputs['interface'])) {
 			foreach ($inputs['interface']['details'] as &$field) {
 				if (strstr($field, '{') !== false) {
-					$matched_macros = (new CMacrosResolverGeneral)->getMacroPositions($field, ['usermacros' => true]);
+					$matched_macros = CMacrosResolverGeneral::getMacroPositions($field, ['usermacros' => true]);
 
 					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
+						// If matching macro is not found, return unresolved macro string.
 						$macro_value = array_key_exists($macro, $macros_posted)
 							? $macros_posted[$macro]
-							: '';
+							: $macro;
 
 						$field = substr_replace($field, $macro_value, $pos, strlen($macro));
 					}
@@ -1296,6 +1310,33 @@ abstract class CControllerPopupItemTest extends CController {
 
 				return false;
 			}
+
+			if ($interface['details']['version'] == SNMP_V2C || $interface['details']['version'] == SNMP_V3) {
+				if (!array_key_exists('max_repetitions', $interface['details'])
+						|| $interface['details']['max_repetitions'] === '') {
+					error(_s('Incorrect value for field "%1$s": %2$s.', _('Max repetition count'), _('cannot be empty')));
+
+					return false;
+				}
+
+				if (!is_numeric($interface['details']['max_repetitions'])) {
+					error(_s('Incorrect value for field "%1$s": %2$s.', _('Max repetition count'), _('a numeric value is expected')));
+
+					return false;
+				}
+
+				if ($interface['details']['max_repetitions'] < 1) {
+					error(_s('Incorrect value for field "%1$s": %2$s.', _('Max repetition count'), _s('value must be no less than "%1$s"', 1)));
+
+					return false;
+				}
+
+				if ($interface['details']['max_repetitions'] > ZBX_MAX_INT32) {
+					error(_s('Incorrect value for field "%1$s": %2$s.', _('Max repetition count'), _s('value must be no greater than "%1$s"', ZBX_MAX_INT32)));
+
+					return false;
+				}
+			}
 		}
 
 		if ($this->items_require_interface[$this->item_type]['address']
