@@ -91,7 +91,7 @@ if (getRequest('parent_discoveryid')) {
 		$hostPrototype = API::HostPrototype()->get([
 			'output' => API_OUTPUT_EXTEND,
 			'selectGroupLinks' => ['groupid'],
-			'selectGroupPrototypes' => ['name'],
+			'selectGroupPrototypes' => ['group_prototypeid', 'name'],
 			'selectTemplates' => ['templateid', 'name'],
 			'selectParentHost' => ['hostid'],
 			'selectMacros' => ['hostmacroid', 'macro', 'value', 'type', 'description'],
@@ -110,14 +110,6 @@ else {
 	access_deny();
 }
 
-$tags = getRequest('tags', []);
-foreach ($tags as $key => $tag) {
-	// remove empty new tag lines
-	if ($tag['tag'] === '' && $tag['value'] === '') {
-		unset($tags[$key]);
-	}
-}
-
 // Remove inherited macros data (actions: 'add', 'update' and 'form').
 $macros = cleanInheritedMacros(getRequest('macros', []));
 
@@ -151,176 +143,132 @@ elseif (isset($_REQUEST['delete']) && isset($_REQUEST['hostid'])) {
 elseif (isset($_REQUEST['clone']) && isset($_REQUEST['hostid'])) {
 	unset($_REQUEST['hostid']);
 
-	if ($macros && in_array(ZBX_MACRO_TYPE_SECRET, array_column($macros, 'type'))) {
-		// Reset macro type and value.
-		$macros = array_map(function($value) {
-			return ($value['type'] == ZBX_MACRO_TYPE_SECRET)
-				? ['value' => '', 'type' => ZBX_MACRO_TYPE_TEXT] + $value
-				: $value;
-		}, $macros);
+	if (hasRequest('group_prototypes')) {
+		foreach ($_REQUEST['group_prototypes'] as &$group_prototype) {
+			unset($group_prototype['group_prototypeid']);
+		}
+		unset($group_prototype);
+	}
 
-		warning(_('The cloned host prototype contains user defined macros with type "Secret text". The value and type of these macros were reset.'));
+	$warnings = [];
+
+	// Reset macro type and value.
+	$secret_macro_reset = false;
+
+	foreach ($macros as &$macro) {
+		if (array_key_exists('allow_revert', $macro) && array_key_exists('value', $macro)) {
+			$macro['deny_revert'] = true;
+
+			unset($macro['allow_revert']);
+		}
+	}
+	unset($macro);
+
+	foreach ($macros as &$macro) {
+		if ($macro['type'] == ZBX_MACRO_TYPE_SECRET && !array_key_exists('value', $macro)) {
+			$macro = [
+				'type' => ZBX_MACRO_TYPE_TEXT,
+				'value' => ''
+			] + $macro;
+
+			$secret_macro_reset = true;
+
+			unset($macro['allow_revert']);
+		}
+	}
+	unset($macro);
+
+	if ($secret_macro_reset) {
+		$warnings[] = _('The cloned host prototype contains user defined macros with type "Secret text". The value and type of these macros were reset.');
 	}
 
 	$macros = array_map(function(array $macro): array {
 		return array_diff_key($macro, array_flip(['hostmacroid']));
 	}, $macros);
 
+	if (CWebUser::getType() != USER_TYPE_SUPER_ADMIN && getRequest('group_links')) {
+		$editable_groups_count = API::HostGroup()->get([
+			'countOutput' => true,
+			'groupids' => getRequest('group_links'),
+			'editable' => true
+		]);
+
+		if ($editable_groups_count != count(getRequest('group_links'))) {
+			$warnings[] = _("The host being cloned belongs to a host group you don't have write permissions to. Non-writable group has been removed from the new host.");
+		}
+	}
+
+	if ($warnings) {
+		if (count($warnings) > 1) {
+			CMessageHelper::setWarningTitle(_('Cloned host parameter values have been modified.'));
+		}
+
+		array_map('CMessageHelper::addWarning', $warnings);
+	}
+
 	$_REQUEST['form'] = 'clone';
 }
 elseif (hasRequest('add') || hasRequest('update')) {
-	DBstart();
-
-	foreach ($macros as &$macro) {
-		unset($macro['discovery_state']);
-	}
-	unset($macro);
-
-	$save_macros = $macros;
-
-	foreach ($save_macros as &$macro) {
-		unset($macro['allow_revert']);
-	}
-	unset($macro);
-
-	if ($hostid == 0 || $hostPrototype['templateid'] == 0) {
-		$newHostPrototype = [
-			'host' => getRequest('host', ''),
-			'name' => (getRequest('name', '') === '') ? getRequest('host', '') : getRequest('name', ''),
+	try {
+		$input = [
+			'host' => getRequest('host', DB::getDefault('hosts', 'host')),
+			'name' => getRequest(getRequest('name', '') === '' ? 'host' : 'name', DB::getDefault('hosts', 'name')),
+			'custom_interfaces' => getRequest('custom_interfaces', DB::getDefault('hosts', 'custom_interfaces')),
 			'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
 			'discover' => getRequest('discover', DB::getDefault('hosts', 'discover')),
-			'groupLinks' => [],
-			'groupPrototypes' => [],
-			'tags' => $tags,
-			'macros' => $save_macros,
-			'templates' => array_merge(getRequest('templates', []), getRequest('add_templates', [])),
-			'custom_interfaces' => getRequest('custom_interfaces', DB::getDefault('hosts', 'custom_interfaces'))
+			'interfaces' => prepareHostPrototypeInterfaces(
+				getRequest('interfaces', []), getRequest('mainInterfaces', [])
+			),
+			'groupLinks' => prepareHostPrototypeGroupLinks(getRequest('group_links', [])),
+			'groupPrototypes' => prepareHostPrototypeGroupPrototypes(getRequest('group_prototypes', [])),
+			'templates' => zbx_toObject(
+				array_merge(getRequest('templates', []), getRequest('add_templates', [])),
+				'templateid'
+			),
+			'tags' => prepareHostPrototypeTags(getRequest('tags', [])),
+			'macros' => prepareHostPrototypeMacros($macros),
+			'inventory_mode' => getRequest('inventory_mode', HOST_INVENTORY_DISABLED)
 		];
 
-		if (hasRequest('inventory_mode')) {
-			$newHostPrototype['inventory_mode'] = getRequest('inventory_mode');
-		}
+		$result = true;
 
-		// API requires 'templateid' property.
-		if ($newHostPrototype['templates']) {
-			$newHostPrototype['templates'] = zbx_toObject($newHostPrototype['templates'], 'templateid');
-		}
+		if (hasRequest('add')) {
+			$host = ['ruleid' => getRequest('parent_discoveryid')] + getSanitizedHostPrototypeFields(
+				['templateid' => 0] + $input
+			);
 
-		// add custom group prototypes
-		foreach (getRequest('group_prototypes', []) as $groupPrototype) {
-			if ($groupPrototype['name'] !== '') {
-				$newHostPrototype['groupPrototypes'][] = $groupPrototype;
+			$response = API::HostPrototype()->create($host);
+
+			if ($response === false) {
+				throw new Exception();
 			}
 		}
 
-		if ($newHostPrototype['custom_interfaces'] == HOST_PROT_INTERFACES_CUSTOM) {
-			$interfaces = getRequest('interfaces', []);
+		if (hasRequest('update')) {
+			$host = ['hostid' => $hostid] + getSanitizedHostPrototypeFields(
+				['templateid' => $hostPrototype['templateid']] + $input
+			);
 
-			foreach ($interfaces as &$interface) {
-				// Process SNMP interface fields.
-				if ($interface['type'] == INTERFACE_TYPE_SNMP) {
-					$interface['details']['bulk'] = array_key_exists('bulk', $interface['details'])
-						? SNMP_BULK_ENABLED
-						: SNMP_BULK_DISABLED;
+			$response = API::HostPrototype()->update($host);
 
-					switch ($interface['details']['version']) {
-						case SNMP_V1:
-						case SNMP_V2C:
-							$interface['details'] = array_intersect_key($interface['details'],
-								array_flip(['version', 'bulk', 'community'])
-							);
-							break;
-
-						case SNMP_V3:
-							$field_names = array_flip(['version', 'bulk', 'contextname', 'securityname',
-								'securitylevel'
-							]);
-
-							if ($interface['details']['securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV) {
-								$field_names += array_flip(['authprotocol', 'authpassphrase']);
-							}
-							elseif ($interface['details']['securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
-								$field_names +=
-									array_flip(['authprotocol', 'authpassphrase', 'privprotocol', 'privpassphrase']);
-							}
-
-							$interface['details'] = array_intersect_key($interface['details'], $field_names);
-							break;
-					}
-				}
-
-				unset($interface['isNew'], $interface['items'], $interface['interfaceid']);
-				$interface['main'] = 0;
+			if ($response === false) {
+				throw new Exception();
 			}
-			unset($interface);
-
-			$main_interfaces = getRequest('mainInterfaces', []);
-
-			foreach (CItem::INTERFACE_TYPES_BY_PRIORITY as $type) {
-				if (array_key_exists($type, $main_interfaces)
-						&& array_key_exists($main_interfaces[$type], $interfaces)) {
-					$interfaces[$main_interfaces[$type]]['main'] = INTERFACE_PRIMARY;
-				}
-			}
-
-			$newHostPrototype['interfaces'] = $interfaces;
 		}
-	}
-	else {
-		$newHostPrototype = [
-			'status' => getRequest('status', HOST_STATUS_NOT_MONITORED),
-			'discover' => getRequest('discover', DB::getDefault('hosts', 'discover'))
-		];
+	} catch (Exception $e) {
+		$result = false;
 	}
 
-	if ($hostid != 0) {
-		$newHostPrototype['hostid'] = $hostid;
-
-		if (!$hostPrototype['templateid']) {
-			// add group prototypes based on existing host groups
-			$groupPrototypesByGroupId = zbx_toHash($hostPrototype['groupLinks'], 'groupid');
-			unset($groupPrototypesByGroupId[0]);
-			foreach (getRequest('group_links', []) as $groupId) {
-				$newHostPrototype['groupLinks'][] = [
-					'groupid' => array_key_exists($groupId, $groupPrototypesByGroupId)
-						? $groupPrototypesByGroupId[$groupId]['groupid']
-						: $groupId
-				];
-
-			}
-		}
-		else {
-			unset($newHostPrototype['groupPrototypes'], $newHostPrototype['groupLinks']);
-		}
-
-		$result = API::HostPrototype()->update($newHostPrototype);
-
-		show_messages($result, _('Host prototype updated'), _('Cannot update host prototype'));
-	}
-	else {
-		$newHostPrototype['ruleid'] = getRequest('parent_discoveryid');
-
-		// add group prototypes based on existing host groups
-		foreach (getRequest('group_links', []) as $groupId) {
-			$newHostPrototype['groupLinks'][] = [
-				'groupid' => $groupId
-			];
-		}
-
-		if (count($newHostPrototype['groupPrototypes']) === 0) {
-			unset($newHostPrototype['groupPrototypes']);
-		}
-
-		$result = API::HostPrototype()->create($newHostPrototype);
-
+	if (hasRequest('add')) {
 		show_messages($result, _('Host prototype added'), _('Cannot add host prototype'));
 	}
-
-	$result = DBend($result);
+	else {
+		show_messages($result, _('Host prototype updated'), _('Cannot update host prototype'));
+	}
 
 	if ($result) {
-		uncheckTableRows($discoveryRule['itemid']);
 		unset($_REQUEST['itemid'], $_REQUEST['form']);
+		uncheckTableRows($discoveryRule['itemid']);
 	}
 }
 elseif ($hostid != 0 && getRequest('action', '') === 'hostprototype.updatediscover') {
@@ -408,7 +356,7 @@ if (hasRequest('form')) {
 		'show_inherited_macros' => getRequest('show_inherited_macros', 0),
 		'readonly' => ($hostid != 0 && $hostPrototype['templateid']),
 		'groups' => [],
-		'tags' => $tags,
+		'tags' => getRequest('tags', []),
 		'context' => getRequest('context'),
 		// Parent discovery rules.
 		'templates' => []
@@ -474,7 +422,8 @@ if (hasRequest('form')) {
 			$data['host_prototype'] = array_merge($data['host_prototype'], $hostPrototype);
 
 			foreach ($data['host_prototype']['macros'] as &$macro) {
-				if ($macro['type'] == ZBX_MACRO_TYPE_SECRET) {
+				if ($macro['type'] == ZBX_MACRO_TYPE_SECRET
+						&& !array_key_exists('deny_revert', $macro) && !array_key_exists('value', $macro)) {
 					$macro['allow_revert'] = true;
 				}
 			}
@@ -572,13 +521,25 @@ if (hasRequest('form')) {
 		'add_templates' => array_map('strval', array_keys($data['host_prototype']['add_templates']))
 	];
 
+	// Editable host groups.
+	$groups_rw = ($data['groups'] && CWebUser::getType() != USER_TYPE_SUPER_ADMIN)
+		? API::HostGroup()->get([
+			'output' => [],
+			'groupids' => array_keys($data['groups']),
+			'editable' => true,
+			'preservekeys' => true
+		])
+		: [];
+
 	$data['groups_ms'] = [];
 
 	foreach ($data['groups'] as $group) {
 		$data['groups_ms'][] = [
 			'id' => $group['groupid'],
 			'name' => $group['name'],
-			'inaccessible' => (array_key_exists('inaccessible', $group) && $group['inaccessible'])
+			'inaccessible' => array_key_exists('inaccessible', $group) && $group['inaccessible'],
+			'disabled' => CWebUser::getType() != USER_TYPE_SUPER_ADMIN
+					&& !array_key_exists($group['groupid'], $groups_rw)
 		];
 	}
 
