@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -31,7 +31,6 @@ class CConfigurationExportBuilder {
 	 */
 	public function __construct() {
 		$this->data['version'] = ZABBIX_EXPORT_VERSION;
-		$this->data['date'] = date(DATE_TIME_FORMAT_SECONDS_XML, time() - date('Z'));
 	}
 
 	/**
@@ -43,6 +42,77 @@ class CConfigurationExportBuilder {
 		return ['zabbix_export' => $this->data];
 	}
 
+	/**
+	 * Build row structure.
+	 *
+	 * @param array  $rule      Validation rule for selected tag in 3rd parameter.
+	 * @param array  $row       Export row.
+	 * @param string $tag       Tag name.
+	 * @param string $main_tag  Main element (for error reporting).
+	 *
+	 * @throws Exception  if row is invalid.
+	 *
+	 * @return mixed
+	 */
+	private static function buildArrayRow(array $rule, array $row, string $tag, string $main_tag) {
+		if (array_key_exists('ex_rules', $rule)) {
+			$parent_rule = array_intersect_key($rule, array_flip(['ex_default', 'default', 'rule']));
+			$rule = call_user_func($rule['ex_rules'], $row);
+			$rule = $parent_rule + $rule;
+		}
+
+		$is_required = (bool) ($rule['type'] & XML_REQUIRED);
+		$is_string = (bool) ($rule['type'] & XML_STRING);
+		$is_array = (bool) ($rule['type'] & XML_ARRAY);
+		$is_indexed_array = (bool) ($rule['type'] & XML_INDEXED_ARRAY);
+		$has_data = array_key_exists($tag, $row);
+
+		if (array_key_exists('ex_default', $rule)) {
+			$default_value = (string) call_user_func($rule['ex_default'], $row);
+		}
+		elseif (array_key_exists('default', $rule)) {
+			$default_value = (string) $rule['default'];
+		}
+		else {
+			$default_value = null;
+		}
+
+		if (!$default_value && !$has_data) {
+			if ($is_required) {
+				throw new Exception(_s('Invalid tag "%1$s": %2$s.', $main_tag, _s('the tag "%1$s" is missing', $tag)));
+			}
+			return null;
+		}
+
+		$value = $has_data ? $row[$tag] : $default_value;
+
+		if (!$is_required && $default_value == $value) {
+			return null;
+		}
+
+		if (($is_indexed_array || $is_array) && $has_data) {
+			$temp_store = self::build($rule, $is_array ? [$value] : $value, $tag);
+
+			return ($is_required || $temp_store) ? $temp_store : null;
+		}
+
+		if ($is_string && $value !== null) {
+			$value = str_replace("\r\n", "\n", $value);
+		}
+
+		if (array_key_exists('in', $rule)) {
+			if (!array_key_exists($value, $rule['in'])) {
+				throw new Exception(_s('Invalid tag "%1$s": %2$s.', $tag,
+					_s('unexpected constant value "%1$s"', $value)
+				));
+			}
+
+			return $rule['in'][$value];
+		}
+
+		return $value;
+	}
+
 	/**
 	 * Build data structure.
 	 *
@@ -52,80 +122,36 @@ class CConfigurationExportBuilder {
 	 *
 	 * @return array
 	 */
-	protected function build(array $schema, array $data, $main_tag = null) {
+	private static function build(array $schema, array $data, string $main_tag) {
 		$n = 0;
 		$result = [];
 
-		$rules = $schema['rules'];
-
 		if ($schema['type'] & XML_INDEXED_ARRAY) {
-			$rules = $schema['rules'][$schema['prefix']]['rules'];
+			$type = $schema['rules'][$schema['prefix']]['type'] & (XML_ARRAY | XML_STRING);
+			$rules = $type == XML_ARRAY
+				? $schema['rules'][$schema['prefix']]['rules']
+				: $schema['rules'][$schema['prefix']];
+		}
+		else {
+			$type = XML_ARRAY;
+			$rules = $schema['rules'];
 		}
 
 		foreach ($data as $row) {
 			$store = [];
-			foreach ($rules as $tag => $val) {
-				$is_required = $val['type'] & XML_REQUIRED;
-				$is_string = $val['type'] & XML_STRING;
-				$is_array = $val['type'] & XML_ARRAY;
-				$is_indexed_array = $val['type'] & XML_INDEXED_ARRAY;
-				$has_data = array_key_exists($tag, $row);
 
-				if (array_key_exists('ex_default', $val)) {
-					$default_value = (string) call_user_func($val['ex_default'], $row);
-				}
-				elseif (array_key_exists('default', $val)) {
-					$default_value = (string) $val['default'];
-				}
-				else {
-					$default_value = null;
-				}
+			if ($type == XML_ARRAY) {
+				foreach ($rules as $tag => $rule) {
+					$value = self::buildArrayRow($rule, $row, $tag, $main_tag);
 
-				if (!$default_value && !$has_data) {
-					if ($is_required) {
-						throw new Exception(_s('Invalid tag "%1$s": %2$s.', $main_tag,
-							_s('the tag "%1$s" is missing', $tag)
-						));
+					if ($value !== null) {
+						$store[$tag] = $value;
 					}
-					continue;
-				}
-
-				$value = $has_data ? $row[$tag] : $default_value;
-
-				if (!$is_required && $has_data && $default_value == $value) {
-					continue;
-				}
-
-				if (array_key_exists('export', $val)) {
-					$store[$tag] = call_user_func($val['export'], $row);
-					continue;
-				}
-
-				if (($is_indexed_array || $is_array) && $has_data) {
-					$temp_store = $this->build($val, $is_array ? [$value] : $value, $tag);
-					if ($is_required || $temp_store) {
-						$store[$tag] = $temp_store;
-					}
-					continue;
-				}
-
-				if ($is_string && $value !== null) {
-					$value = str_replace("\r\n", "\n", $value);
-				}
-
-				if (array_key_exists('in', $val)) {
-					if (!array_key_exists($value, $val['in'])) {
-						throw new Exception(_s('Invalid tag "%1$s": %2$s.', $tag,
-							_s('unexpected constant value "%1$s"', $value)
-						));
-					}
-
-					$store[$tag] = $val['in'][$value];
-				}
-				else {
-					$store[$tag] = $value;
 				}
 			}
+			else {
+				$store = str_replace("\r\n", "\n", $row);
+			}
 
 			if ($schema['type'] & XML_INDEXED_ARRAY) {
 				$result[$n++] = $store;
@@ -139,15 +165,27 @@ class CConfigurationExportBuilder {
 	}
 
 	/**
-	 * Format groups.
+	 * Format template groups.
 	 *
 	 * @param array $schema  Tag schema from validation class.
 	 * @param array $groups  Export data.
 	 */
-	public function buildGroups(array $schema, array $groups) {
+	public function buildTemplateGroups(array $schema, array $groups) {
 		$groups = $this->formatGroups($groups);
 
-		$this->data['groups'] = $this->build($schema, $groups, 'groups');
+		$this->data['template_groups'] = self::build($schema, $groups, 'template_groups');
+	}
+
+	/**
+	 * Format host groups.
+	 *
+	 * @param array $schema  Tag schema from validation class.
+	 * @param array $groups  Export data.
+	 */
+	public function buildHostGroups(array $schema, array $groups) {
+		$groups = $this->formatGroups($groups);
+
+		$this->data['host_groups'] = self::build($schema, $groups, 'host_groups');
 	}
 
 	/**
@@ -160,7 +198,7 @@ class CConfigurationExportBuilder {
 	public function buildTemplates(array $schema, array $templates, array $simple_triggers) {
 		$templates = $this->formatTemplates($templates, $simple_triggers);
 
-		$this->data['templates'] = $this->build($schema, $templates, 'templates');
+		$this->data['templates'] = self::build($schema, $templates, 'templates');
 	}
 
 	/**
@@ -173,7 +211,7 @@ class CConfigurationExportBuilder {
 	public function buildHosts(array $schema, array $hosts, array $simple_triggers) {
 		$hosts = $this->formatHosts($hosts, $simple_triggers);
 
-		$this->data['hosts'] = $this->build($schema, $hosts, 'hosts');
+		$this->data['hosts'] = self::build($schema, $hosts, 'hosts');
 	}
 
 	/**
@@ -185,7 +223,7 @@ class CConfigurationExportBuilder {
 	public function buildTriggers(array $schema, array $triggers) {
 		$triggers = $this->formatTriggers($triggers);
 
-		$this->data['triggers'] = $this->build($schema, $triggers, 'triggers');
+		$this->data['triggers'] = self::build($schema, $triggers, 'triggers');
 	}
 
 	/**
@@ -197,7 +235,7 @@ class CConfigurationExportBuilder {
 	public function buildGraphs(array $schema, array $graphs) {
 		$graphs = $this->formatGraphs($graphs);
 
-		$this->data['graphs'] = $this->build($schema, $graphs, 'graphs');
+		$this->data['graphs'] = self::build($schema, $graphs, 'graphs');
 	}
 
 	/**
@@ -209,22 +247,24 @@ class CConfigurationExportBuilder {
 	public function buildMediaTypes(array $schema, array $media_types) {
 		$media_types = $this->formatMediaTypes($media_types);
 
-		$this->data['media_types'] = $this->build($schema, $media_types, 'media_types');
+		$this->data['media_types'] = self::build($schema, $media_types, 'media_types');
 	}
 
 	/**
 	 * Separate simple triggers.
 	 *
 	 * @param array $triggers
+	 * @param array $unlink_itemids
 	 *
 	 * @return array
 	 */
-	public function extractSimpleTriggers(array &$triggers) {
+	public function extractSimpleTriggers(array &$triggers, array $unlink_itemids) {
 		$simple_triggers = [];
 
 		foreach ($triggers as $triggerid => $trigger) {
 			if (count($trigger['items']) == 1 && $trigger['items'][0]['type'] != ITEM_TYPE_HTTPTEST
-					&& $trigger['items'][0]['templateid'] == 0) {
+					&& ($trigger['items'][0]['templateid'] == 0
+						|| in_array($trigger['items'][0]['templateid'], $unlink_itemids))) {
 				$simple_triggers[] = $trigger;
 				unset($triggers[$triggerid]);
 			}
@@ -245,12 +285,22 @@ class CConfigurationExportBuilder {
 		CArrayHelper::sort($templates, ['host']);
 
 		foreach ($templates as $template) {
+			$vendor = [];
+
+			if ($template['vendor_name'] !== '' && $template['vendor_version'] !== '') {
+				$vendor = [
+					'name' => $template['vendor_name'],
+					'version' => $template['vendor_version']
+				];
+			}
+
 			$result[] = [
 				'uuid' => $template['uuid'],
 				'template' => $template['host'],
 				'name' => $template['name'],
 				'description' => $template['description'],
-				'groups' => $this->formatGroups($template['groups']),
+				'vendor' => $vendor,
+				'groups' => $this->formatGroups($template['templategroups']),
 				'items' => $this->formatItems($template['items'], $simple_triggers),
 				'discovery_rules' => $this->formatDiscoveryRules($template['discoveryRules']),
 				'httptests' => $this->formatHttpTests($template['httptests']),
@@ -290,7 +340,7 @@ class CConfigurationExportBuilder {
 				'ipmi_username' => $host['ipmi_username'],
 				'ipmi_password' => $host['ipmi_password'],
 				'templates' => $this->formatTemplateLinkage($host['parentTemplates']),
-				'groups' => $this->formatGroups($host['groups']),
+				'groups' => $this->formatGroups($host['hostgroups']),
 				'interfaces' => $this->formatHostInterfaces($host['interfaces']),
 				'items' => $this->formatItems($host['items'], $simple_triggers),
 				'discovery_rules' => $this->formatDiscoveryRules($host['discoveryRules']),
@@ -373,7 +423,7 @@ class CConfigurationExportBuilder {
 			];
 		}
 
-		$this->data['maps'] = $this->build($schema, $this->data['maps'], 'maps');
+		$this->data['maps'] = self::build($schema, $this->data['maps'], 'maps');
 	}
 
 	/**
@@ -386,8 +436,8 @@ class CConfigurationExportBuilder {
 
 		CArrayHelper::sort($media_types, ['name']);
 
-		foreach ($media_types as $media_type) {
-			$result[] = [
+		foreach ($media_types as $i => $media_type) {
+			$result[$i] = [
 				'name' => $media_type['name'],
 				'type' => $media_type['type'],
 				'smtp_server' => $media_type['smtp_server'],
@@ -417,6 +467,10 @@ class CConfigurationExportBuilder {
 				'description' => $media_type['description'],
 				'message_templates' => self::formatMediaTypeMessageTemplates($media_type['message_templates'])
 			];
+
+			if ($media_type['type'] == MEDIA_TYPE_EMAIL) {
+				$result[$i] += ['provider' => $media_type['provider']];
+			}
 		}
 
 		return $result;
@@ -432,14 +486,16 @@ class CConfigurationExportBuilder {
 	 * @return array|string
 	 */
 	private static function formatMediaTypeParameters(array $media_type) {
-		if ($media_type['type'] == MEDIA_TYPE_EXEC) {
-			return $media_type['exec_params'];
-		}
+		switch ($media_type['type']) {
+			case MEDIA_TYPE_EXEC:
+				CArrayHelper::sort($media_type['parameters'], ['sortorder']);
 
-		if ($media_type['type'] == MEDIA_TYPE_WEBHOOK) {
-			CArrayHelper::sort($media_type['parameters'], ['name']);
+				return array_values($media_type['parameters']);
 
-			return array_values($media_type['parameters']);
+			case MEDIA_TYPE_WEBHOOK:
+				CArrayHelper::sort($media_type['parameters'], ['name']);
+
+				return array_values($media_type['parameters']);
 		}
 
 		return [];
@@ -580,7 +636,7 @@ class CConfigurationExportBuilder {
 				'password' => $discoveryRule['password'],
 				'publickey' => $discoveryRule['publickey'],
 				'privatekey' => $discoveryRule['privatekey'],
-				'filter' => $discoveryRule['filter'],
+				'filter' => self::formatLldFilter($discoveryRule['filter']),
 				'lifetime' => $discoveryRule['lifetime'],
 				'description' => $discoveryRule['description'],
 				'item_prototypes' => $this->formatItems($discoveryRule['itemPrototypes'], $simple_trigger_prototypes),
@@ -591,7 +647,7 @@ class CConfigurationExportBuilder {
 				'timeout' => $discoveryRule['timeout'],
 				'url' => $discoveryRule['url'],
 				'query_fields' => $discoveryRule['query_fields'],
-				'parameters' => $discoveryRule['parameters'],
+				'parameters' => self::formatItemParameters($discoveryRule['parameters']),
 				'posts' => $discoveryRule['posts'],
 				'status_codes' => $discoveryRule['status_codes'],
 				'follow_redirects' => $discoveryRule['follow_redirects'],
@@ -606,11 +662,15 @@ class CConfigurationExportBuilder {
 				'ssl_key_password' => $discoveryRule['ssl_key_password'],
 				'verify_peer' => $discoveryRule['verify_peer'],
 				'verify_host' => $discoveryRule['verify_host'],
-				'lld_macro_paths' => $discoveryRule['lld_macro_paths'],
+				'lld_macro_paths' => self::formatLldMacroPaths($discoveryRule['lld_macro_paths']),
 				'preprocessing' => self::formatPreprocessingSteps($discoveryRule['preprocessing']),
-				'overrides' => $discoveryRule['overrides']
+				'overrides' => self::formatLldOverrides($discoveryRule['overrides'])
 			];
 
+			if (!$data['filter']['conditions']) {
+				unset($data['filter']);
+			}
+
 			if (isset($discoveryRule['interface_ref'])) {
 				$data['interface_ref'] = $discoveryRule['interface_ref'];
 			}
@@ -648,6 +708,70 @@ class CConfigurationExportBuilder {
 		return $result;
 	}
 
+	/**
+	 * Format the LLD filter.
+	 *
+	 * @param array $filter
+	 *
+	 * @return array
+	 */
+	private static function formatLldFilter(array $filter): array {
+		$filter['conditions'] = sortLldRuleFilterConditions($filter['conditions'], $filter['evaltype']);
+
+		return $filter;
+	}
+
+	/**
+	 * Format the LLD macro paths.
+	 *
+	 * @param array $lld_macro_paths
+	 *
+	 * @return array
+	 */
+	private static function formatLldMacroPaths(array $lld_macro_paths): array {
+		CArrayHelper::sort($lld_macro_paths, ['lld_macro']);
+
+		return array_values($lld_macro_paths);
+	}
+
+
+	/**
+	 * Format the LLD overrides contained in a discovery rule.
+	 *
+	 * @param array $overrides
+	 *
+	 * @return array
+	 */
+	private static function formatLldOverrides(array $overrides): array {
+		CArrayHelper::sort($overrides, ['step']);
+
+		foreach ($overrides as &$override) {
+			$override['filter'] = self::formatLldFilter($override['filter']);
+
+			unset($override['filter']['eval_formula']);
+
+			if (!$override['filter']['conditions']) {
+				unset($override['filter']);
+			}
+
+			CArrayHelper::sort($override['operations'], ['operationobject', 'operator', 'value']);
+
+			foreach ($override['operations'] as &$operation) {
+				if (array_key_exists('tags', $operation)) {
+					CArrayHelper::sort($operation['tags'], ['tag', 'value']);
+				}
+
+				if (array_key_exists('templates', $operation)) {
+					CArrayHelper::sort($operation['templates'], ['name']);
+				}
+			}
+			unset($operation);
+		}
+		unset($override);
+
+		return array_values($overrides);
+	}
+
 	/**
 	 * Format preprocessing steps.
 	 *
@@ -682,6 +806,8 @@ class CConfigurationExportBuilder {
 		order_result($httptests, 'name');
 
 		foreach ($httptests as $httptest) {
+			CArrayHelper::sort($httptest['variables'], ['name']);
+
 			$result[] = [
 				'uuid' => $httptest['uuid'],
 				'name' => $httptest['name'],
@@ -721,6 +847,8 @@ class CConfigurationExportBuilder {
 		order_result($httpsteps, 'no');
 
 		foreach ($httpsteps as $httpstep) {
+			CArrayHelper::sort($httpstep['variables'], ['name']);
+
 			$result[] = [
 				'name' => $httpstep['name'],
 				'url' => $httpstep['url'],
@@ -762,7 +890,29 @@ class CConfigurationExportBuilder {
 	protected function formatGraphs(array $graphs) {
 		$result = [];
 
-		CArrayHelper::sort($graphs, ['name']);
+		usort($graphs, static function (array $graph_a, array $graph_b): int {
+			$comparison = strnatcasecmp($graph_a['name'], $graph_b['name']);
+
+			if ($comparison != 0) {
+				return $comparison;
+			}
+
+			$graph_a_items = [];
+			$graph_b_items = [];
+
+			foreach ($graph_a['gitems'] as $gitem) {
+				$graph_a_items[] = $gitem['itemid'];
+			}
+
+			foreach ($graph_b['gitems'] as $gitem) {
+				$graph_b_items[] = $gitem['itemid'];
+			}
+
+			CArrayHelper::sort($graph_a_items, ['host']);
+			CArrayHelper::sort($graph_b_items, ['host']);
+
+			return strnatcasecmp(reset($graph_a_items)['host'], reset($graph_b_items)['host']);
+		});
 
 		foreach ($graphs as $graph) {
 			$data = [
@@ -839,14 +989,14 @@ class CConfigurationExportBuilder {
 	 *
 	 * @return array
 	 */
-	protected function formatGroupLinks(array $groupLinks) {
+	protected function formatGroupLinks(array $group_links) {
 		$result = [];
 
-		CArrayHelper::sort($groupLinks, ['name']);
+		CArrayHelper::sort($group_links, ['name']);
 
-		foreach ($groupLinks as $groupLink) {
+		foreach ($group_links as $group_link) {
 			$result[] = [
-				'group' => $groupLink['groupid']
+				'group' => $group_link
 			];
 		}
 
@@ -917,6 +1067,7 @@ class CConfigurationExportBuilder {
 				'opdata' => $trigger['opdata'],
 				'correlation_mode' => $trigger['correlation_mode'],
 				'correlation_tag' => $trigger['correlation_tag'],
+				'url_name' => $trigger['url_name'],
 				'url' => $trigger['url'],
 				'status' => $trigger['status'],
 				'priority' => $trigger['priority'],
@@ -1064,7 +1215,7 @@ class CConfigurationExportBuilder {
 				'timeout' => $item['timeout'],
 				'url' => $item['url'],
 				'query_fields' => $item['query_fields'],
-				'parameters' => $item['parameters'],
+				'parameters' => self::formatItemParameters($item['parameters']),
 				'posts' => $item['posts'],
 				'status_codes' => $item['status_codes'],
 				'follow_redirects' => $item['follow_redirects'],
@@ -1138,6 +1289,19 @@ class CConfigurationExportBuilder {
 		return $result;
 	}
 
+	/**
+	 * Format item parameters.
+	 *
+	 * @param array $parameters
+	 *
+	 * @return array
+	 */
+	private static function formatItemParameters(array $parameters): array {
+		CArrayHelper::sort($parameters, ['name']);
+
+		return array_values($parameters);
+	}
+
 	/**
 	 * Format macros.
 	 *
@@ -1265,7 +1429,7 @@ class CConfigurationExportBuilder {
 	protected function formatWidgets(array $widgets) {
 		$result = [];
 
-		CArrayHelper::sort($widgets, ['name']);
+		CArrayHelper::sort($widgets, ['x', 'y']);
 
 		foreach ($widgets as $widget) {
 			$result[] = [
@@ -1293,7 +1457,7 @@ class CConfigurationExportBuilder {
 	protected function formatWidgetFields(array $fields) {
 		$result = [];
 
-		CArrayHelper::sort($fields, ['type']);
+		self::sortWidgetFields($fields);
 
 		foreach ($fields as $field) {
 			$result[] = [
@@ -1306,6 +1470,58 @@ class CConfigurationExportBuilder {
 		return $result;
 	}
 
+	/**
+	 * Sorts widget fields taking into account expanded objects.
+	 *
+	 * @param array $fields
+	 */
+	private static function sortWidgetFields(array &$fields): void {
+		usort($fields, static function(array $widget_field_a, array $widget_field_b): int {
+			$comparison = strnatcasecmp($widget_field_a['name'], $widget_field_b['name']);
+
+			if ($comparison != 0) {
+				return $comparison;
+			}
+
+			$comparison = strnatcasecmp($widget_field_a['type'], $widget_field_b['type']);
+
+			if ($comparison != 0) {
+				return $comparison;
+			}
+
+			switch ($widget_field_a['type']) {
+				case ZBX_WIDGET_FIELD_TYPE_HOST:
+					$value_fields = ['host'];
+					break;
+
+				case ZBX_WIDGET_FIELD_TYPE_ITEM:
+				case ZBX_WIDGET_FIELD_TYPE_ITEM_PROTOTYPE:
+					$value_fields = ['host', 'key'];
+					break;
+
+				case ZBX_WIDGET_FIELD_TYPE_GRAPH:
+				case ZBX_WIDGET_FIELD_TYPE_GRAPH_PROTOTYPE:
+					$value_fields = ['host', 'name'];
+					break;
+
+				default:
+					return strnatcasecmp($widget_field_a['value'], $widget_field_b['value']);
+			}
+
+			foreach ($value_fields as $value_field) {
+				$comparison = strnatcasecmp($widget_field_a['value'][$value_field],
+					$widget_field_b['value'][$value_field]
+				);
+
+				if ($comparison != 0) {
+					return $comparison;
+				}
+			}
+
+			return 0;
+		});
+	}
+
 	/**
 	 * Format graph items.
 	 *
