@@ -1,7 +1,7 @@
-<?php declare(strict_types = 1);
+<?php declare(strict_types = 0);
 /*
 ** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -34,12 +34,16 @@ class CButtonExport extends CList {
 				->removeAttribute('name')
 				->removeAttribute('value')
 				->addClass(ZBX_STYLE_BTN_ALT)
-				->onClick('var $_form = jQuery(this).closest("form");'.
-					// Save the original form action.
-					'if (!$_form.data("action")) {'.
-						'$_form.data("action", $_form.attr("action"));'.
-					'}'.
-					'$_form.attr("action", '.json_encode(
+				->onClick('const form = this.closest("form");'.
+					/*
+					 * Save the original form action.
+					 * Function getAttribute()/setAttribute() is used instead of .action, because there are many
+					 * buttons with name 'action' and .action selects these buttons.
+					 */
+					'if (!form.dataset.action) {
+						form.dataset.action = form.getAttribute("action");
+					}'.
+					'form.setAttribute("action", '. json_encode(
 						(new CUrl('zabbix.php'))
 							->setArgument('action', $action)
 							->setArgument('format', CExportWriterFactory::YAML)
@@ -47,7 +51,7 @@ class CButtonExport extends CList {
 							->getUrl()
 					).');'
 				),
-			(new CButton('export', '&#8203;'))
+			(new CButton('export', ZWSPACE()))
 				->addClass(ZBX_STYLE_BTN_ALT)
 				->addClass(ZBX_STYLE_BTN_TOGGLE_CHEVRON)
 				->setMenuPopup([
