@@ -347,7 +347,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		// Replace macros to value.
 		foreach (array_keys($macros) as $hostid) {
 			foreach ($data[$hostid] as &$text) {
-				$matched_macros = $this->getMacroPositions($text, $types);
+				$matched_macros = self::getMacroPositions($text, $types);
 
 				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 					$text = substr_replace($text, $macros[$hostid][$macro], $pos, strlen($macro));
@@ -447,8 +447,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 					if (array_key_exists($data['f_num'], $functionids)) {
 						$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = [
 							'token' => $token,
-							'function' => $data['function'],
-							'parameters' => $data['parameters']
+							'macrofunc' => $data['macrofunc']
 						];
 					}
 				}
@@ -507,7 +506,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		foreach ($macro_values as $triggerid => $foo) {
 			$trigger = &$triggers[$triggerid];
 
-			$matched_macros = $this->getMacroPositions($trigger['description'], $types);
+			$matched_macros = self::getMacroPositions($trigger['description'], $types);
 
 			foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 				if (array_key_exists($macro, $macro_values[$triggerid])) {
@@ -611,8 +610,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				if (array_key_exists($data['f_num'], $functionids)) {
 					$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = [
 						'token' => $token,
-						'function' => $data['function'],
-						'parameters' => $data['parameters']
+						'macrofunc' => $data['macrofunc']
 					];
 				}
 			}
@@ -659,7 +657,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			$trigger = &$triggers[$triggerid];
 
 			foreach ($options['sources'] as $source) {
-				$matched_macros = $this->getMacroPositions($trigger[$source], $types);
+				$matched_macros = self::getMacroPositions($trigger[$source], $types);
 
 				if ($options['html']) {
 					$macro_string = [];
@@ -790,8 +788,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 			if (array_key_exists($data['f_num'], $functionids)) {
 				$macros['item'][$functionids[$data['f_num']]][$data['macro']][] = [
 					'token' => $token,
-					'function' => $data['function'],
-					'parameters' => $data['parameters']
+					'macrofunc' => $data['macrofunc']
 				];
 			}
 		}
@@ -832,7 +829,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 		$types = $this->transformToPositionTypes($types);
 
-		$matched_macros = $this->getMacroPositions($trigger['url'], $types);
+		$matched_macros = self::getMacroPositions($trigger['url'], $types);
 
 		$url = $trigger['url'];
 		foreach (array_reverse($matched_macros, true) as $pos => $macro) {
@@ -981,7 +978,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 		$types = $this->transformToPositionTypes($types);
 
-		$matched_macros = $this->getMacroPositions($trigger['url_name'], $types);
+		$matched_macros = self::getMacroPositions($trigger['url_name'], $types);
 
 		$url_name = $trigger['url_name'];
 		foreach (array_reverse($matched_macros, true) as $pos => $macro) {
@@ -1918,7 +1915,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 		// Replace macros to value.
 		foreach (array_keys($macro_values) as $key) {
 			foreach ($options['sources'] as $source) {
-				$matched_macros = $this->getMacroPositions($data[$key][$source], $types);
+				$matched_macros = self::getMacroPositions($data[$key][$source], $types);
 
 				foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 					$data[$key][$source] =
@@ -1966,9 +1963,11 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 				: $usermacros_data['macros'];
 		}
 
+		$types = $this->transformToPositionTypes($types);
+
 		// Replace macros to value.
 		foreach ($macro_values as $key => $macros) {
-			$function = $this->resolveFunctionMacros($functions[$key]['function_string'], $macros);
+			$function = $this->resolveFunctionMacros($functions[$key]['function_string'], $macros, $types);
 			$function = substr_replace($function, TRIGGER_QUERY_PLACEHOLDER, $functions[$key]['function_query_pos'], 8);
 			$functions[$key]['parameter'] = substr($function, strlen($functions[$key]['function']) + 1, -1);
 		}
@@ -3022,7 +3021,7 @@ class CMacrosResolver extends CMacrosResolverGeneral {
 
 			foreach ($scripts as &$fields) {
 				foreach ($fields as &$value) {
-					$matched_macros = $this->getMacroPositions($value, $types);
+					$matched_macros = self::getMacroPositions($value, $types);
 
 					foreach (array_reverse($matched_macros, true) as $pos => $macro) {
 						if (array_key_exists($macro, $macro_values[$eventid])) {
