@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2023 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -29,19 +29,10 @@ class CUrlFactory {
 	 * @var array
 	 */
 	protected static $contextConfigs = [
-		'actionconf.php' => [
-			'remove' => ['actionid']
-		],
-		'applications.php' => [
-			'remove' => ['applicationid']
-		],
 		'disc_prototypes.php' => [
 			'remove' => ['itemid'],
 			'add' => ['hostid', 'parent_discoveryid']
 		],
-		'discoveryconf.php' => [
-			'remove' => ['druleid']
-		],
 		'graphs.php' => [
 			'remove' => ['graphid'],
 			'add' => ['hostid', 'parent_discoveryid']
@@ -54,28 +45,12 @@ class CUrlFactory {
 			'remove' => ['hostid'],
 			'add' => ['parent_discoveryid']
 		],
-		'hostgroups.php' => [
-			'remove' => ['groupid']
-		],
-		'hosts.php' => [
-			'remove' => ['hostid']
-		],
 		'httpconf.php' => [
 			'remove' => ['httptestid']
 		],
 		'items.php' => [
 			'remove' => ['itemid']
 		],
-		'maintenance.php' => [
-			'remove' => ['maintenanceid']
-		],
-		'screenconf.php' => [
-			'remove' => ['screenid'],
-			'add' => ['templateid']
-		],
-		'slideconf.php' => [
-			'remove' => ['slideshowid']
-		],
 		'sysmaps.php' => [
 			'remove' => ['sysmapid']
 		],
@@ -90,9 +65,6 @@ class CUrlFactory {
 			'remove' => ['triggerid'],
 			'add' => ['hostid']
 		],
-		'usergrps.php' => [
-			'remove' => ['usrgrpid']
-		],
 		'__default' => [
 			'remove' => ['cancel', 'form', 'delete']
 		]
